@@ -5,10 +5,8 @@ short.summary <- function(object) {
 
     # catch FAKE run
     FAKE <- FALSE
-    if(!is.null(object@Model@control$optim.method)) {
-        if(tolower(object@Model@control$optim.method) == "none") {
-            FAKE <- TRUE
-        }
+    if(object@Options$optim.method == "none") {
+        FAKE <- TRUE
     }
 
     # Convergence or not?
@@ -327,393 +325,6 @@ function(object, header       = TRUE,
     }
 })
 
-# old summary (<0.5-19)
-summary2 <- function(object, estimates=TRUE, fit.measures=FALSE, 
-                     standardized=FALSE, 
-                     rsquare=FALSE, std.nox=FALSE, modindices=FALSE) {
-
-    if(std.nox) standardized <- TRUE
-
-    # always print the 'short' summary
-    short.summary(object)
-
-    # only if requested, the fit measures
-    if(fit.measures) {
-        if(object@Options$test == "none") {
-            warning("lavaan WARNING: fit measures not available if test = \"none\"\n\n")
-        } else if(object@optim$npar > 0L && !object@optim$converged) {
-            warning("lavaan WARNING: fit measures not available if model did not converge\n\n")
-        } else {
-            print.fit.measures( fitMeasures(object, fit.measures="default") )
-        }
-    }
-
-
-    if(estimates) {
-
-    # main part: parameter estimates
-    cat("\nParameter estimates:\n\n")
-    t0.txt <- sprintf("  %-40s", "Information")
-    tmp.txt <- object@Options$information
-    t1.txt <- sprintf("  %10s", paste(toupper(substring(tmp.txt,1,1)), 
-			 	     substring(tmp.txt,2), sep=""))
-    cat(t0.txt, t1.txt, "\n", sep="")
-    t0.txt <- sprintf("  %-31s", "Standard Errors")
-    tmp.txt <- object@Options$se
-    t1.txt <- sprintf("  %19s", paste(toupper(substring(tmp.txt,1,1)),  
-                                      substring(tmp.txt,2), sep=""))
-    cat(t0.txt, t1.txt, "\n", sep="")
-    if(object@Options$se == "bootstrap") {
-        t0.txt <- sprintf("  %-40s", "Number of requested bootstrap draws")
-        t1.txt <- sprintf("  %10i", object@Options$bootstrap)
-        cat(t0.txt, t1.txt, "\n", sep="")
-        t0.txt <- sprintf("  %-40s", "Number of successful bootstrap draws")
-        t1.txt <- sprintf("  %10i", NROW(object@boot$coef))
-        cat(t0.txt, t1.txt, "\n", sep="")
-    }
-    cat("\n")
-
-    # local print function
-    print.estimate <- function(name="ERROR", i=1, z.stat=TRUE) {
-       
-        # cut name if (still) too long
-        name <- strtrim(name, width=13L)
-
-        if(!standardized) {
-            if(is.na(se[i])) {
-                txt <- sprintf("    %-13s %9.3f %8.3f\n", name, est[i], se[i])
-            } else if(se[i] == 0) {
-                txt <- sprintf("    %-13s %9.3f\n", name, est[i])
-            } else if(est[i]/se[i] > 9999.999) {
-                txt <- sprintf("    %-13s %9.3f %8.3f\n", name, est[i], se[i])
-            } else if(!z.stat) {
-                txt <- sprintf("    %-13s %9.3f %8.3f\n", name, est[i], se[i])
-            } else {
-                z <- est[i]/se[i]
-                pval <- 2 * (1 - pnorm( abs(z) ))
-                txt <- sprintf("    %-13s %9.3f %8.3f %8.3f %8.3f\n",
-                               name, est[i], se[i], z, pval)
-            }
-        } else {
-            if(is.na(se[i])) {
-                txt <- sprintf("    %-13s %9.3f %8.3f                   %8.3f %8.3f\n", name, est[i], se[i], est.std[i], est.std.all[i])
-            } else if(se[i] == 0) {
-                txt <- sprintf("    %-13s %9.3f                            %8.3f %8.3f\n", name, est[i], est.std[i], est.std.all[i])
-            } else if(est[i]/se[i] > 9999.999) {
-                txt <- sprintf("    %-13s %9.3f %8.3f                   %8.3f %8.3f\n", name, est[i], se[i], est.std[i], est.std.all[i])
-            } else if(!z.stat) {
-                txt <- sprintf("    %-13s %9.3f %8.3f                   %8.3f %8.3f\n", name, est[i], se[i], est.std[i], est.std.all[i])
-            } else {
-                z <- est[i]/se[i]
-                pval <- 2 * (1 - pnorm( abs(z) ))
-                txt <- sprintf("    %-13s %9.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n",
-                               name, est[i], se[i], z, pval, est.std[i], est.std.all[i])
-            }
-        }
-        cat(txt)
-    }
-
-    est <- lav_object_inspect_est(object)
-    se  <- lav_object_inspect_se(object)
-    if(rsquare || standardized) {
-        est.std    <- standardize.est.lv(object)
-        if(std.nox) {
-            est.std.all <- standardize.est.all.nox(object, est.std=est.std)
-        } else {
-            est.std.all <- standardize.est.all(object, est.std=est.std)
-        }
-    } 
-
-    for(g in 1:object@Data@ngroups) {
-        ov.names <- vnames(object@ParTable, "ov", group=g)
-        lv.names <- vnames(object@ParTable, "lv", group=g)
-
-        # group header
-        if(object@Data@ngroups > 1) {
-            if(g > 1) cat("\n\n")
-            cat("Group ", g, 
-                " [", object@Data@group.label[[g]], "]:\n\n", sep="")
-        }
-
-        # estimates header
-        if(!standardized) {
-            cat("                   Estimate  Std.err  Z-value  P(>|z|)\n")
-        } else {
-            if(std.nox) {
-                cat("                   Estimate  Std.err  Z-value  P(>|z|)   Std.lv  Std.nox\n")
-            }
-            else {
-                cat("                   Estimate  Std.err  Z-value  P(>|z|)   Std.lv  Std.all\n")
-            }
-        }
-
-        makeNames <- function(NAMES, LABELS) {
-            multiB <- FALSE
-            if(any(nchar(NAMES) != nchar(NAMES, "bytes")))
-                multiB <- TRUE
-            if(any(nchar(LABELS) != nchar(LABELS, "bytes")))
-                multiB <- TRUE
-            # labels?
-            l.idx <- which(nchar(LABELS) > 0L)
-            if(length(l.idx) > 0L) {
-                if(!multiB) {
-                    LABELS <- abbreviate(LABELS, 4)
-                    LABELS[l.idx] <- paste(" (", LABELS[l.idx], ")", sep="")
-                    MAX.L <- max(nchar(LABELS))
-                    NAMES <- abbreviate(NAMES, minlength = (13 - MAX.L), 
-                                        strict = TRUE)
-                } else {
-                    # do not abbreviate anything (eg in multi-byte locales)
-                    MAX.L <- 4L
-                }
-                NAMES <- sprintf(paste("%-", (13 - MAX.L), "s%", MAX.L, "s",
-                                       sep=""), NAMES, LABELS)
-            } else {
-                if(!multiB) {
-                    NAMES <- abbreviate(NAMES, minlength = 13, strict = TRUE)
-                } else {
-                    NAMES <- sprintf(paste("%-", 13, "s", sep=""), NAMES)
-                }
-            }
-
-            NAMES
-        }
-
-        NAMES <- object@ParTable$rhs
-
-        # 1a. indicators ("=~") (we do show dummy indicators)
-        mm.idx <- which( object@ParTable$op == "=~" & 
-                        !object@ParTable$lhs %in% ov.names &
-                         object@ParTable$group == g)
-        if(length(mm.idx)) {
-            cat("Latent variables:\n")
-            lhs.old <- ""
-            NAMES[mm.idx] <- makeNames(  object@ParTable$rhs[mm.idx],
-                                       object@ParTable$label[mm.idx])
-            for(i in mm.idx) {
-                lhs <- object@ParTable$lhs[i]
-                if(lhs != lhs.old) cat("  ", lhs, " =~\n", sep="")
-                print.estimate(name=NAMES[i], i)
-                lhs.old <- lhs
-            }
-            cat("\n")
-        }
-
-        # 1b. formative/composites ("<~")
-        fm.idx <- which( object@ParTable$op == "<~" &
-                         object@ParTable$group == g)
-        if(length(fm.idx)) {
-            cat("Composites:\n")
-            lhs.old <- ""
-            NAMES[fm.idx] <- makeNames(  object@ParTable$rhs[fm.idx],
-                                       object@ParTable$label[fm.idx])
-            for(i in fm.idx) {
-                lhs <- object@ParTable$lhs[i]
-                if(lhs != lhs.old) cat("  ", lhs, " <~\n", sep="")
-                print.estimate(name=NAMES[i], i)
-                lhs.old <- lhs
-            }
-            cat("\n")
-        }
-
-        # 2. regressions
-        eqs.idx <- which(object@ParTable$op == "~" & object@ParTable$group == g)
-        if(length(eqs.idx) > 0) {
-            cat("Regressions:\n")
-            lhs.old <- ""
-            NAMES[eqs.idx] <- makeNames(  object@ParTable$rhs[eqs.idx],
-                                        object@ParTable$label[eqs.idx])
-            for(i in eqs.idx) {
-                lhs <- object@ParTable$lhs[i]
-                if(lhs != lhs.old) cat("  ", lhs, " ~\n", sep="")
-                print.estimate(name=NAMES[i], i)
-                lhs.old <- lhs
-            }
-            cat("\n")
-        }
-
-        # 3. covariances
-        cov.idx <- which(object@ParTable$op == "~~" & 
-                         !object@ParTable$exo &
-                         object@ParTable$lhs != object@ParTable$rhs &
-                         object@ParTable$group == g)
-        if(length(cov.idx) > 0) {
-            cat("Covariances:\n")
-            lhs.old <- ""
-            NAMES[cov.idx] <- makeNames(  object@ParTable$rhs[cov.idx],
-                                        object@ParTable$label[cov.idx])
-            for(i in cov.idx) {
-                lhs <- object@ParTable$lhs[i]
-                if(lhs != lhs.old) cat("  ", lhs, " ~~\n", sep="")
-                print.estimate(name=NAMES[i], i)
-                lhs.old <- lhs
-            }
-            cat("\n")
-        }
-
-        # 4. intercepts/means
-        #ord.names <- vnames(object@ParTable, type="ov.ord", group=g)
-        int.idx <- which(object@ParTable$op == "~1" & 
-                         #!object@ParTable$lhs %in% ord.names &
-                         !object@ParTable$exo &
-                         object@ParTable$group == g)
-        if(length(int.idx) > 0) {
-            cat("Intercepts:\n")
-            NAMES[int.idx] <- makeNames(  object@ParTable$lhs[int.idx],
-                                        object@ParTable$label[int.idx])
-            for(i in int.idx) {
-                print.estimate(name=NAMES[i], i)
-            }
-            cat("\n")
-        }
-
-        # 4b thresholds
-        th.idx <- which(object@ParTable$op == "|" &
-                        object@ParTable$group == g)
-        if(length(th.idx) > 0) {
-            cat("Thresholds:\n")
-            NAMES[th.idx] <- makeNames(  paste(object@ParTable$lhs[th.idx],
-                                               "|",
-                                               object@ParTable$rhs[th.idx],
-                                               sep=""),
-                                         object@ParTable$label[th.idx])
-            for(i in th.idx) {
-                print.estimate(name=NAMES[i], i)
-            }
-            cat("\n")
-        }
-
-        # 5. (residual) variances
-        var.idx <- which(object@ParTable$op == "~~" &
-                         !object@ParTable$exo &
-                         object@ParTable$lhs == object@ParTable$rhs &
-                         object@ParTable$group == g)
-        if(length(var.idx) > 0) {
-            cat("Variances:\n")
-            NAMES[var.idx] <- makeNames(  object@ParTable$rhs[var.idx],
-                                        object@ParTable$label[var.idx])
-            for(i in var.idx) {
-                if(object@Options$mimic == "lavaan") {
-                    print.estimate(name=NAMES[i], i, z.stat=FALSE)
-                } else {
-                    print.estimate(name=NAMES[i], i, z.stat=TRUE)
-                }
-            }
-            cat("\n")
-        }
-
-        # 6. latent response scales
-        delta.idx <- which(object@ParTable$op == "~*~" &
-                         object@ParTable$group == g)
-        if(length(delta.idx) > 0) {
-            cat("Scales y*:\n")
-            NAMES[delta.idx] <- makeNames(  object@ParTable$rhs[delta.idx],
-                                            object@ParTable$label[delta.idx])
-            for(i in delta.idx) {
-                print.estimate(name=NAMES[i], i, z.stat=TRUE)
-            }
-            cat("\n")
-        }
-
-        # 7. group weight
-        group.idx <- which(object@ParTable$lhs == "group" &
-                           object@ParTable$op == "%" &
-                           object@ParTable$group == g)
-        if(length(group.idx) > 0) {
-            cat("Group weight:\n")
-            NAMES[group.idx] <- makeNames(  object@ParTable$rhs[group.idx],
-                                            object@ParTable$label[group.idx])
-            for(i in group.idx) {
-                print.estimate(name=NAMES[i], i, z.stat=TRUE)
-            }
-            cat("\n")
-        }
-
-    } # ngroups
-
-    # 6. variable definitions
-    def.idx <- which(object@ParTable$op == ":=")
-    if(length(def.idx) > 0) {
-        if(object@Data@ngroups > 1) cat("\n")
-        cat("Defined parameters:\n")
-        NAMES[def.idx] <- makeNames(  object@ParTable$lhs[def.idx], "")
-        for(i in def.idx) {
-            print.estimate(name=NAMES[i], i)
-        }
-        cat("\n")
-    }
-
-    # 7. constraints
-    cin.idx <- which((object@ParTable$op == "<" | 
-                      object@ParTable$op == ">"))
-    ceq.idx <- which(object@ParTable$op == "==" & object@ParTable$user == 1L)
-    if(length(cin.idx) > 0L || length(ceq.idx) > 0L) {
-        # set small negative values to zero, to avoid printing " -0.000"
-        slack <- ifelse(abs(est) < 1e-5, 0, est)
-        #slack[cin.idx] <- object@Model@cin.function(object@optim$x)
-        #slack[ceq.idx] <- object@Model@ceq.function(object@optim$x)
-       
-        if(object@Data@ngroups > 1 && length(def.idx) == 0L) cat("\n")
-        cat("Constraints:                               Slack (>=0)\n")
-        for(i in c(cin.idx,ceq.idx)) {
-            lhs <- object@ParTable$lhs[i]
-             op <- object@ParTable$op[i]
-            rhs <- object@ParTable$rhs[i]
-            if(rhs == "0" && op == ">") {
-                con.string <- paste(lhs, " - 0", sep="")
-            } else if(rhs == "0" && op == "<") {
-                con.string <- paste(rhs, " - (", lhs, ")", sep="")
-            } else if(rhs != "0" && op == ">") {
-                con.string <- paste(lhs, " - (", rhs, ")", sep="")
-            } else if(rhs != "0" && op == "<") {
-                con.string <- paste(rhs, " - (", lhs, ")", sep="")
-            } else if(rhs == "0" && op == "==") {
-                con.string <- paste(lhs, " - 0", sep="")
-            } else if(rhs != "0" && op == "==") {
-                con.string <- paste(lhs, " - (", rhs, ")", sep="")
-            }
-            con.string <- abbreviate(con.string, 41, strict = TRUE)
-            txt <- sprintf("    %-41s %8.3f\n", 
-                           con.string, slack[i])
-            cat(txt)
-        }   
-        cat("\n")
-    }
-
-    } # parameter estimates
-
-
-    # R-square?
-    if(rsquare) {
-        r2 <- lav_object_inspect_rsquare(object, est.std.all=est.std.all,
-                  drop.list.single.group = FALSE, add.labels = TRUE,
-                  add.class = FALSE)
-        for(g in 1:object@Data@ngroups) {
-            if(object@Data@ngroups > 1) {
-                cat("R-Square Group ", g, " [", 
-                    object@Data@group.label[[g]], "]",
-                    ":\n\n", sep="")
-            } else {
-                cat("R-Square:\n\n")
-            } 
-            for(i in 1:length(r2[[g]])) {
-                t1.txt <- sprintf("    %-13s %9.3f\n", names(r2[[g]])[i], 
-                                  r2[[g]][i])
-                cat(t1.txt)
-            }
-            if(g < object@Data@ngroups) cat("\n")
-        }
-    }
-
-    # modification indices?
-    if(modindices) {
-        cat("Modification Indices:\n\n")
-        print( modificationIndices(object, standardized=TRUE) )
-    }
-
-}
-
-
 
 setMethod("coef", "lavaan",
 function(object, type="free", labels=TRUE) {
@@ -745,7 +356,9 @@ standardizedSolution <- standardizedsolution <- function(object,
                                                          pvalue = TRUE,
                                                          remove.eq = TRUE,
                                                          remove.ineq = TRUE,
-                                                         remove.def = FALSE) {
+                                                         remove.def = FALSE,
+                                                         GLIST = NULL,
+                                                         est   = NULL) {
 
     stopifnot(type %in% c("std.all", "std.lv", "std.nox"))
 
@@ -754,6 +367,15 @@ standardizedSolution <- standardizedsolution <- function(object,
         zstat <- pvalue <- FALSE
     }
 
+    # no se if class is not lavaan
+    if(class(object) != "lavaan") {
+        if(missing(se) || !se) {
+            se <- FALSE
+            zstat <- FALSE
+            pvalue <- FALSE
+        }
+    }
+
     PARTABLE <- inspect(object, "list")
     free.idx <- which(PARTABLE$free > 0L)
     LIST <- PARTABLE[,c("lhs", "op", "rhs")]
@@ -763,11 +385,11 @@ standardizedSolution <- standardizedsolution <- function(object,
 
     # add std and std.all columns
     if(type == "std.lv") {
-        LIST$est.std     <- standardize.est.lv(object)
+        LIST$est.std     <- standardize.est.lv(object, est = est, GLIST = GLIST)
     } else if(type == "std.all") {
-        LIST$est.std <- standardize.est.all(object)
+        LIST$est.std <- standardize.est.all(object, est = est, GLIST = GLIST)
     } else if(type == "std.nox") {
-        LIST$est.std <- standardize.est.all.nox(object)
+        LIST$est.std <- standardize.est.all.nox(object, est = est, GLIST = GLIST)
     }
 
     if(object@Options$se != "none" && se) {
@@ -857,6 +479,19 @@ parameterEstimates <- parameterestimates <- function(object,
                                                      rsquare = FALSE,
                                                      add.attributes = FALSE) {
 
+    if("lavaan.fsr" %in% class(object)) {
+        return(object$PE)
+    }
+
+    # no se if class is not lavaan
+    if(class(object) != "lavaan") {
+        if(missing(se) || !se) {
+            se <- FALSE
+            zstat <- FALSE
+            pvalue <- FALSE
+        }
+    }
+
     # check fmi
     if(fmi) {
         if(inherits(object, "lavaanList")) {
@@ -891,6 +526,16 @@ parameterEstimates <- parameterestimates <- function(object,
     if(!is.null(PARTABLE$user)) {
         LIST$user <- PARTABLE$user
     }
+    if(!is.null(PARTABLE$block)) {
+        LIST$block <- PARTABLE$block
+    } else {
+        LIST$block <- rep(1L, length(LIST$lhs))
+    }
+    if(!is.null(PARTABLE$level)) {
+        LIST$level <- PARTABLE$level
+    } else {
+        LIST$level <- rep(1L, length(LIST$lhs))
+    }
     if(!is.null(PARTABLE$group)) {
         LIST$group <- PARTABLE$group
     } else {
@@ -1090,7 +735,7 @@ parameterEstimates <- parameterestimates <- function(object,
         r2 <- lavTech(object, "rsquare", add.labels = TRUE)
         NAMES <- unlist(lapply(r2, names)); nel <- length(NAMES)
         R2 <- data.frame( lhs = NAMES, op = rep("r2", nel), rhs = NAMES,
-                          group = rep(1:length(r2), sapply(r2, length)),
+                          block = rep(1:length(r2), sapply(r2, length)),
                           est = unlist(r2), stringsAsFactors = FALSE )
         LIST <- lav_partable_merge(pt1 = LIST, pt2 = R2, warn = FALSE)
     }
@@ -1098,8 +743,9 @@ parameterEstimates <- parameterestimates <- function(object,
     # fractional missing information (if estimator="fiml")
     if(fmi) {
         SE.orig <- LIST$se
-        COV <- object@implied$cov
-        MEAN <- object@implied$mean
+        lavmodel <- object@Model; implied <- object@implied
+        COV <- if(lavmodel@conditional.x) implied$res.cov else implied$cov
+        MEAN <- if(lavmodel@conditional.x) implied$res.int else implied$mean
 
         # provide rownames
         for(g in 1:object@Data@ngroups)
@@ -1132,9 +778,18 @@ parameterEstimates <- parameterestimates <- function(object,
         LIST$fmi <- 1-(SE.step2*SE.step2/(SE.orig*SE.orig))
     }
 
+    # if single level, remove level column
+    if(object@Data@nlevels == 1L) LIST$level <- NULL
+
     # if single group, remove group column
     if(object@Data@ngroups == 1L) LIST$group <- NULL
 
+    # if single everything, remove block column
+    if(object@Data@nlevels == 1L &&
+       object@Data@ngroups == 1L) {
+        LIST$block <- NULL
+    }
+
     # if no user-defined labels, remove label column
     if(sum(nchar(object@ParTable$label)) == 0L) LIST$label <- NULL
 
@@ -1175,8 +830,13 @@ parameterEstimates <- parameterestimates <- function(object,
         attr(LIST, "information") <- object@Options$information
         attr(LIST, "se") <- object@Options$se
         attr(LIST, "group.label") <- object@Data@group.label
+        attr(LIST, "level.label") <- object@Data@level.label
         attr(LIST, "bootstrap") <- object@Options$bootstrap
         attr(LIST, "bootstrap.successful") <- bootstrap.successful
+        attr(LIST, "missing") <- object@Options$missing
+        attr(LIST, "observed.information") <- 
+            object@Options$observed.information
+        attr(LIST, "h1.information") <- object@Options$h1.information
         # FIXME: add more!!
     } else {
         LIST$exo <- NULL
@@ -1235,43 +895,9 @@ function(object, type = "moments", labels=TRUE) {
         return( lavPredict(object, type = "ov", label = labels) )
     }
 
-    G <- object@Data@ngroups
-    ov.names <- object@Data@ov.names
-
-    OUT <- vector("list", length=G)
-    for(g in 1:G) {
-        OUT[[g]]$cov  <- object@implied$cov[[g]]
-        if(labels) 
-            rownames(OUT[[g]]$cov) <- colnames(OUT[[g]]$cov) <- ov.names[[g]]
-        class(OUT[[g]]$cov) <- c("lavaan.matrix.symmetric", "matrix")
-
-        #if(object@Model@meanstructure) {
-            OUT[[g]]$mean <- as.numeric(object@implied$mean[[g]])
-            if(labels) names(OUT[[g]]$mean) <- ov.names[[g]]
-            class(OUT[[g]]$mean) <- c("lavaan.vector", "numeric")
-        #}
-
-        if(object@Model@categorical) {
-            OUT[[g]]$th <- as.numeric(object@implied$th[[g]])
-            if(length(object@Model@num.idx[[g]]) > 0L) {
-                NUM.idx <- which(object@Model@th.idx[[g]] == 0)
-                OUT[[g]]$th <- OUT[[g]]$th[ -NUM.idx ]
-            }
-            if(labels) {
-                names(OUT[[g]]$th) <-
-                    vnames(object@ParTable, type="th", group=g)
-            }
-            class(OUT[[g]]$th) <- c("lavaan.vector", "numeric")
-        }
-    }
-
-    if(G == 1) {
-        OUT <- OUT[[1]]
-    } else {
-        names(OUT) <- unlist(object@Data@group.label)
-    }
-
-    OUT
+    lav_object_inspect_implied(object,
+               add.labels = labels, add.class = TRUE,
+               drop.list.single.group = TRUE)
 })
 
 
@@ -1292,7 +918,7 @@ function(object, labels = TRUE, remove.duplicated = FALSE) {
         stop("lavaan ERROR: vcov not available if se=\"none\"")
     }
 
-    VarCov <- lav_object_inspect_vcov(lavobject = object,
+    VarCov <- lav_object_inspect_vcov(object,
                                       add.labels = labels,
                                       add.class = TRUE,
                                       remove.duplicated = remove.duplicated)
